<?php

namespace App\ContentModule\Presenters;

use App\Model\UserManager;
use App\Forms\UserForms;
use Nette\Application\UI\Form;
use Nette\Utils\ArrayHash;

/**
 * Zpracovává vykreslování uživatelské sekce.
 * @package App\ContentModule\Presenters
 */
class UserPresenter extends BaseContentPresenter {

    /** @var UserForms Továrnička na uživatelské formuláře. */
    private $userFormsFactory;

    /** @var array Společné instrukce pro přihlašovací a registrační formuláře. */
    private $instructions;
    private $userManager;

    public function __construct(UserManager $userManager, UserForms $userForms) {
        parent::__construct();
        $this->userManager = $userManager;
        $this->userFormsFactory = $userForms;
    }

    /** Volá se před každou akcí presenteru a inicializuje společné proměnné. */
    public function startup() {
        parent::startup();
        $this->instructions = array(
            'message' => null,
            'redirection' => null,
            'redirectionAdmin' => ':Content:Administration:',
            'redirectionMember' => ':Content:User:'
        );
    }

    /** Přesměrování, pokud je uživatel již přihlášen. */
    public function actionLogin() {
        if ($this->getUser()->isLoggedIn()) {
            if ($this->getUser()->isInRole('admin')) {
                $this->redirect(':Content:Administration:');
            } else if ($this->getUser()->isInRole('member')) {
                $this->redirect(':Content:User:');
            }
        }
    }

    /** Odhlášení uživatele. */
    public function actionLogout() {
        $this->getUser()->logout();
        $this->redirect($this->loginPresenter);
    }

    /** Profil uživatele. */
    public function renderDefault() {
        $identity = $this->getUser()->getIdentity();
        if ($identity) { $this->template->username = $identity->getData()['username']; }
    }

    /**
     * Vykresluje editaci detailu uživatele.
     */
    public function actionDetail() {
        $identity = $this->getUser()->getIdentity();
        if ($identity) { $this->template->userdata = $identity->getData(); }
        if ($identity) {
            ($userdata = $identity->getData()) 
                    ? $this['detailForm']->setDefaults($userdata) 
                    : $this->flashMessage('Uživatel není přihlášen.');
        }
    }

    /**
     * Vrátí formulář pro editaci uživatele.
     */
    protected function createComponentDetailForm() {
        $form = new Form;
        $form->addHidden('user_id');
        $form->addText('name', 'Jméno')->setRequired();
        $form->addText('surname', 'Příjmení')->setRequired();
        $form->addInteger('phone', 'Telefon')->setRequired();
        $form->addText('email', 'Email')->setRequired();
        $form->addText('street', 'Ulice')->setRequired();
        $form->addInteger('number', 'Čp')->setRequired();
        $form->addText('city', 'Město')->setRequired();
        $form->addText('zip_code', 'PSČ')->setRequired();
        $form->addSubmit('submit', 'Uložit detail');
        $form->onSuccess[] = [$this, 'detailFormSucceeded'];
        return $form;
    }

    /**
     * Funkce se vykonaná při úspěsném odeslání formuláře; zpracuje hodnoty formuláře.
     * @param Form $form formulář editoru
     * @param ArrayHash $values odeslané hodnoty formuláře
     */
    public function detailFormSucceeded($form, $values) {
        $this->userManager->saveUser($values);
        $this->flashMessage('Detaily byly úspěšně uložen.');
    }

    /**
     * Vrací komponentu přihlašovacího formuláře z továrničky.
     * @return Form přihlašovací formulář
     */
    protected function createComponentLoginForm() {
        $this->instructions['message'] = 'Byl jste úspěšně přihlášen.';
        return $this->userFormsFactory->createLoginForm(ArrayHash::from($this->instructions));
    }

    /**
     * Vrací komponentu registračního formuláře z továrničky.
     * @return Form registrační formulář
     */
    protected function createComponentRegisterForm() {
        $this->instructions['message'] = 'Byl jste úspěšně zaregistrován.';
        return $this->userFormsFactory->createRegisterForm(ArrayHash::from($this->instructions));
    }

}
