<?php

namespace App\Model;

use Nette\Database\UniqueConstraintViolationException;
use Nette\Security\AuthenticationException;
use Nette\Security\IAuthenticator;
use Nette\Security\Identity;
use Nette\Security\Passwords;
use Nette\Utils\Arrays;

/**
 * Správce uživatelů.
 * @package App\Model
 */
class UserManager extends BaseManager implements IAuthenticator {

    /** Konstanty pro manipulaci s modelem. */
    const
            TABLE_NAME = 'user',
            TABLE_NAME_ADDRESS = 'address',
            COLUMN_ID = 'user_id',
            COLUMN_USERNAME = 'username',
            COLUMN_PASSWORD_HASH = 'password',
            COLUMN_ROLE = 'role',
            COLUMN_NAME = 'name',
            COLUMN_SURNAME = 'surname',
            COLUMN_PHONE = 'phone',
            COLUMN_EMAIL = 'email',
            COLUMN_ADDRESSID = 'address_id',
            COLUMN_STREET = 'street',
            COLUMN_NUMBER = 'number',
            COLUMN_CITY = 'city',
            COLUMN_ZIPCODE = 'zip_code';

    /**
     * Přihlásí uživatele do systému.
     * @param array $credentials       jméno a heslo uživatele
     * @return Identity identitu přihlášeného uživatele pro další manipulaci
     * @throws AuthenticationException Jestliže došlo k chybě při prihlašování, např. špatné heslo nebo uživatelské
     *                                 jméno.
     */
    public function authenticate(array $credentials) {
        list($username, $password) = $credentials; // Extrahuje potřebné parametry.
        // Vykoná dotaz nad databází a vrátí první řádek výsledku nebo false, pokud uživatel neexistuje.
        $user = $this->database->table(self::TABLE_NAME)->where(self::COLUMN_USERNAME, $username)->fetch();

        // Ověření uživatele.
        if (!$user) {
            // Vyhodí výjimku, pokud uživatel neexituje.
            throw new AuthenticationException('Zadané uživatelské jméno neexistuje.', self::IDENTITY_NOT_FOUND);
        } elseif (!Passwords::verify($password, $user[self::COLUMN_PASSWORD_HASH])) { // Ověří heslo.
            // Vyhodí výjimku, pokud je heslo špatně.
            throw new AuthenticationException('Zadané heslo není správně.', self::INVALID_CREDENTIAL);
        } elseif (Passwords::needsRehash($user[self::COLUMN_PASSWORD_HASH])) { // Zjistí, jestli heslo potřebuje rehashovat.
            // Rehashuje heslo.
            $user->update(array(self::COLUMN_PASSWORD_HASH => Passwords::hash($password)));
        }

        // Příprava uživatelských dat.
        $userData = $user->toArray(); // Extrahuje uživatelská data.
        unset($userData[self::COLUMN_PASSWORD_HASH]); // Odstraní položku hesla z uživatelských dat (kvůli bezpečnosti).
        Arrays::insertAfter($userData, 'address_id', ['street' => $user->address->street,
            'number' => $user->address->number, 'city' => $user->address->city,
            'zip_code' => $user->address->zip_code]);

        // Vrátí novou identitu přihlášeného uživatele.
        return new Identity($user[self::COLUMN_ID], $user[self::COLUMN_ROLE], $userData);
    }

    /**
     * Registruje nového uživatele do systému.
     * @param string $username uživatelské jméno
     * @param string $password heslo
     * @throws DuplicateNameException Jestliže uživatel s daným jménem již existuje.
     */
    public function register($username, $password) {
        try {
            // Pokusí se vložit nového uživatele do databáze.
            $this->database->table(self::TABLE_NAME)->insert(array(
                self::COLUMN_USERNAME => $username,
                self::COLUMN_PASSWORD_HASH => Passwords::hash($password),
            ));
        } catch (UniqueConstraintViolationException $e) {
            // Vyhodí výjimku, pokud uživatel s daným jménem již existuje.
            throw new DuplicateNameException;
        }
    }

    /**
     * Uloží detail uživatele
     */
    public function saveUser($user) {
        if ($user[self::COLUMN_ID]) {
            $address = $this->database->table(self::TABLE_NAME_ADDRESS)->
                            where(self::COLUMN_STREET . ' ? AND '
                                    . self::COLUMN_NUMBER . ' ? AND '
                                    . self::COLUMN_CITY . ' ? AND '
                                    . self::COLUMN_ZIPCODE . ' ?', 
                                    $user->street, $user->number, $user->city, $user->zip_code)->fetch();
            if ($address == null) {
                $this->database->table(self::TABLE_NAME_ADDRESS)->insert(array(
                    self::COLUMN_STREET => $user->street,
                    self::COLUMN_NUMBER => $user->number,
                    self::COLUMN_CITY => $user->city,
                    self::COLUMN_ZIPCODE => $user->zip_code
                ));
                $address = $this->database->table(self::TABLE_NAME_ADDRESS)->
                                where(self::COLUMN_STREET . ' ? AND '
                                        . self::COLUMN_NUMBER . ' ? AND '
                                        . self::COLUMN_CITY . ' ? AND '
                                        . self::COLUMN_ZIPCODE . ' ?', 
                                        $user->street, $user->number, $user->city, $user->zip_code)->fetch();
            }
            $this->database->table(self::TABLE_NAME)->
                    where(self::COLUMN_ID, $user[self::COLUMN_ID])->
                    update([
                        self::COLUMN_NAME => $user->name,
                        self::COLUMN_SURNAME => $user->surname,
                        self::COLUMN_PHONE => $user->phone,
                        self::COLUMN_EMAIL => $user->email,
                        self::COLUMN_ADDRESSID => $address->address_id
            ]);
        }
    }

}

/**
 * Výjimka pro duplicitní uživatelské jméno.
 * @package App\Model
 */
class DuplicateNameException extends AuthenticationException {

    /** Konstruktor s definicích výchozí chybové zprávy. */
    public function __construct() {
        parent::__construct();
        $this->message = 'Uživatel s tímto jménem je již zaregistrovaný.';
    }

}
